<?php
namespace SAV\SavLibraryPlus\Managers;

use \TYPO3\CMS\Core\Utility\GeneralUtility;

/***************************************************************
*  Copyright notice
*
*  (c) 2011 Laurent Foulloy <yolf.typo3@orange.fr>
*  All rights reserved
*
*  This script is part of the TYPO3 project. The TYPO3 project is
*  free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  The GNU General Public License can be found at
*  http://www.gnu.org/copyleft/gpl.html.
*
*  This script is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  This copyright notice MUST APPEAR in all copies of the script!
***************************************************************/

/**
 * Additional header manager.
 *
 * @package SavLibraryPlus
 * @version $ID:$
 */

class AdditionalHeaderManager {

  /**
   * Array of javaScript code used for the view
   *
   * @var array
   */
  protected static $javaScript = array();  
  
  /**
   * Adds a cascading style Sheet
   *
   * @param string $key
   * @param string $cascadingStyleSheet
   *
   * @return none
   */
  public static function addCascadingStyleSheet($cascadingStyleSheet) {
		// Bug #22273 is solved for version 6.x and not yet backported to versions 4.5, 4.6, 4.7.	
		if (version_compare(TYPO3_version, '6.0', '<')) { 	
	  	if(empty($GLOBALS['TSFE']->additionalHeaderData[$cascadingStyleSheet])) {
	  		$GLOBALS['TSFE']->additionalHeaderData[$cascadingStyleSheet] = self::getCascadingStyleSheetLink($cascadingStyleSheet);
	  	}
		} else {
			$GLOBALS['TSFE']->getPageRenderer()->addCssFile($cascadingStyleSheet);
		}
  }

  /**
   * gets the cascading style Sheet link
   *
   * @param string $cascadingStyleSheet
   *
   * @return string
   */
  protected static function getCascadingStyleSheetLink($cascadingStyleSheet) {
  	$cascadingStyleSheetLink = '<link rel="stylesheet" type="text/css" href="' . $cascadingStyleSheet . '" />' . chr(10);
  	return $cascadingStyleSheetLink;
  }

  /**
   * Adds a javaScript file
   *
   * @param string $javaScriptFileName
   *
   * @return none
   */
  public static function addJavaScriptFile($javaScriptFileName) {
		// Bug #22273 is solved for version 6.x and not yet backported to versions 4.5, 4.6, 4.7.
		if (version_compare(TYPO3_version, '6.0', '<')) { 	
	  	if(empty($GLOBALS['TSFE']->additionalHeaderData[$javaScriptFileName])) {
	  		$GLOBALS['TSFE']->additionalHeaderData[$javaScriptFileName] = '<script type="text/javascript" src="' . $javaScriptFileName . '"></script>';
	  	}
		} else {
			$GLOBALS['TSFE']->getPageRenderer()->addJsFile($javaScriptFileName);
		}
  }  

  /**
   * Adds a javaScript inline code
   *
   * @param string $javaScriptFileName
   *
   * @return none
   */
  public static function addJavaScriptInlineCode($key, $javaScriptInlineCode) {
		// Bug #22273 is solved for version 6.x and not yet backported to versions 4.5, 4.6, 4.7.
		if (version_compare(TYPO3_version, '6.0', '<')) { 	
	  	if(empty($GLOBALS['TSFE']->additionalHeaderData[$key])) {
	  		$GLOBALS['TSFE']->additionalHeaderData[$key] = $javaScriptInlineCode;
	  	}
		} else {
			$GLOBALS['TSFE']->getPageRenderer()->addJsInlineCode($key, $javaScriptInlineCode);			
		}
  }    
  
  /**
   * Adds the javaScript header
   *
   * @param none
   *
   * @return none
   */
  public static function addAdditionalJavaScriptHeader() { 
  	if (count(self::$javaScript) > 0) {
  		// Bug #22273 is solved for version 6.x and not yet backported to versions 4.5, 4.6, 4.7.
			if (version_compare(TYPO3_version, '6.0', '<')) { 	
	  		$GLOBALS['TSFE']->additionalHeaderData[\SAV\SavLibraryPlus\Controller\AbstractController::LIBRARY_NAME] = '';	  		
	  	} 		
  		if (count(self::$javaScript['selectAll']) > 0) {
	  		$javaScriptFileName = \TYPO3\CMS\Core\Utility\ExtensionManagementUtility::siteRelPath(\SAV\SavLibraryPlus\Controller\AbstractController::LIBRARY_NAME) . 'Resources/Private/JavaScript/sav_library_plus.js';	
  			self::addJavaScriptFile($javaScriptFileName);				
  		}
  		// Bug #22273 is solved for version 6.x and not yet backported to versions 4.5, 4.6, 4.7.
  		if (version_compare(TYPO3_version, '6.0', '<')) { 	
	    	$GLOBALS['TSFE']->additionalHeaderData[\SAV\SavLibraryPlus\Controller\AbstractController::LIBRARY_NAME. '_JS'] = self::getJavaScriptHeader();            
  		} else {
				$GLOBALS['TSFE']->getPageRenderer()->addJsInlineCode(\SAV\SavLibraryPlus\Controller\AbstractController::LIBRARY_NAME, self::getJavaScriptHeader());
  		}
  	}
  }
   
	/**
   * Adds javaScript to a given key
   *
   * @param $key string The key
   * @param $javaScript array The javaScript
   *
   * @return none
   */
  public static function addJavaScript($key, $javaScript = NULL) {
  	
    if (!is_array(self::$javaScript[$key])) {
      self::$javaScript[$key] = array();
    }
    self::$javaScript[$key][] = $javaScript;   
  }

  /**
   * Gets the javaScript for a given key
   *
   * @param $key string The key
   *
   * @return string the javaScript
   */
  protected static function getJavaScript($key) {
  	if (!empty(self::$javaScript[$key]) && is_array(self::$javaScript[$key])) {
    	return implode(chr(10) . '    ', self::$javaScript[$key]);
  	} else {
  		return '';
  	}
  }
  
  /**
   * Returns the javaScript Header
   *
   * @param none
   *
   * @return string The javaScript Header
   */
  protected static function getJavaScriptHeader() {
    $javaScript = array();

    $javaScript[] = '';
    
  	// Bug #22273 is solved for version 6.x and not yet backported to versions 4.5, 4.6, 4.7.
  	if (version_compare(TYPO3_version, '6.0', '<')) { 	    
	    $javaScript[] = '<script type="text/javascript">';
	    $javaScript[] = '/*<![CDATA[*/';
  	}
  	
    $javaScript[] = '  ' . self::getJavaScript('documentChanged'); 
    $javaScript[] = '  function checkIfRteChanged(x) {';
    $javaScript[] = '    if (RTEarea[x].editor.plugins.UndoRedo.instance.undoPosition>0) {';
    $javaScript[] = '      document.changed = TRUE;';
    $javaScript[] = '    }';
    $javaScript[] = '  }';
    $javaScript[] = '  function submitIfChanged(x) {';
    $javaScript[] = '    ' . self::getJavaScript('checkIfRteChanged');
    $javaScript[] = '    if (document.changed) {';
    $javaScript[] = '      if (confirm("' . \SAV\SavLibraryPlus\Controller\FlashMessages::translate('warning.save') . '"))	{';
    $javaScript[] = '        update(x);';
    $javaScript[] = '        return FALSE;';
    $javaScript[] = '      }';
    $javaScript[] = '      return TRUE;';
    $javaScript[] = '    }';
    $javaScript[] = '    return TRUE;';
    $javaScript[] = '  }';
    $javaScript[] = '  function update(x) {';
    $javaScript[] = '    ' . self::getJavaScript('rteUpdate');
    $javaScript[] = '    ' . self::getJavaScript('selectAll');
    $javaScript[] = '    return TRUE;';
    $javaScript[] = '  }';
    
  	// Bug #22273 is solved for version 6.x and not yet backported to versions 4.5, 4.6, 4.7.
  	if (version_compare(TYPO3_version, '6.0', '<')) { 	     
	    $javaScript[] = '/*]]>*/';
	    $javaScript[] = '</script>';
  	}
  	
    return implode(chr(10), $javaScript);
  }   
  
}

?>
